import sys

#Let's define some useful strings

digistr = "DigiKeyboard."

def digidelay(integer):
	if integer == 0:
		return ""
	return digistr + ("delay(%d)" % integer) + ";\n\tDigiKeyboard.sendKeyStroke(0);\n\t"

def digiprint(string, defdel):
	return digidelay(defdel) + digistr + ("println(\"%s\")" % string) + ";\n\t"

def keypress(keys, defdel):
	return digidelay(defdel) + digistr + ("sendKeyStroke(%s)" % keys) + ";\n\t"

def repeat(prev, integer, defdel):
	endstr = ("for (i=0; i<%s; i++) {\n\t\t" % str(integer-1)) + digidelay(defdel) + prev + "}\n\t"
	return endstr

def modconvert(string):
	if string in ("GUI", "WINDOWS"):
		return "MOD_GUI_LEFT"
	elif string in ("CONTROL", "CTRL"):
		return "MOD_CTRL_LEFT"
	elif string == "ALT":
		return "MOD_ALT_LEFT"
	elif string == "SHIFT":
		return "MOD_SHIFT_LEFT"

def parseblock(seq,ind):
	endstr = ""
	pos = 0
	defdel = 0
	while pos < len(seq[ind]):
		if seq[ind][pos][1] == 'RESERVED':
			if seq[ind][pos][0] == "REM":
				pos += 1
				if seq[ind][pos][1] == "STR":
					endstr += ("// " + seq[ind][pos][0] + "\n")
					break
				else:
					break
			if seq[ind][pos][0] in ("DEFAULT_DELAY","DEFAULTDELAY"):
				pos += 1
				if seq[ind][pos][1] == "INT":
					defdel = int(seq[ind][pos][0])
					break
				else:
					break
			if seq[ind][pos][0] == "DELAY":
				pos += 1
				if seq[ind][pos][1] == "INT":
					endstr += digidelay(int(seq[ind][pos][0]))
					break
				else:
					break
			if seq[ind][pos][0] ==  "STRING":
				pos += 1
				if seq[ind][pos][1] == "STR":
					endstr += digiprint(seq[ind][pos][0], defdel)
					pos += 1
					break
				else:
					break
			if seq[ind][pos][0] == "REPLAY":
				pos += 1
				repcount = 0
				if seq[ind][pos][1] == "INT":
					repcount = int(seq[ind][pos][0])
				endstr += repeat(parseblock(seq,ind-1),int(seq[ind][pos][0]), defdel)
				break
		if seq[ind][pos][1] == 'KEY':
			keystr = "KEY_" + seq[ind][pos][0]
			endstr += keypress(keystr,defdel)
			break
		if seq[ind][pos][1] == 'MODKEY':
			mkeystr = modconvert(seq[ind][pos][0])
			keystr = ""
			for t in seq[ind][pos+1:]:
				if t[1] == "KEY":
					keystr += ("KEY_" + t[0] + ",")
				elif t[1] == "MODKEY":
					mkeystr += (" | " + modconvert(t[0]))
				else:
					sys.stderr.write("Illegal operation on line %d" % ind)
			if not keystr:
				keystr = "0,"
			tstr = keystr + " " + mkeystr
			endstr += keypress(tstr, defdel)
			break
	return endstr



import sys
from ducklex import *
from dparser import *

banner = "// DigiDuck code courtesy of uslurper. https://github.com/uslurper\n// This program currently does not allow the BREAK key, since I couldn't find it in the USB HID spec.\n// Feel free to modify the output code to suit your needs, but I'm leaving the compiler as it is.\n\n"
begstr = """#include "DigiKeyboard.h"\n\n#define KEY_TAB 43\n#define KEY_DOWN 81\n#define KEY_DELETE 42\n#define KEY_PRINTSCREEN 70\n#define KEY_SCROLLLOCK 71\n#define KEY_INSERT 73\n#define KEY_PAUSE 72\n#define KEY_HOME 74\n#define KEY_PAGEUP 75\n#define KEY_END 77\n#define KEY_PAGEDOWN 78\n#define KEY_RIGHTARROW 79\n#define KEY_RIGHT 79\n#define KEY_DOWNARROW 81\n#define KEY_LEFTARROW 80\n#define KEY_UP 82\n#define KEY_UPARROW 82\n#define KEY_NUMLOCK 83\n#define KEY_CAPSLOCK 57\n\nvoid setup() {\n\n\tDigiKeyboard.delay(5000);\n\tDigiKeyboard.sendKeyStroke(0);\n\t"""
ending = "\n}\n\nvoid loop() {\n\n}"

def usage():
	print "DIGIDUCK: Ducky Script to Digispark compiler"
	print
	print
	print "Usage: digiduck [duckyscript.txt] [outputfile.txt]"
	print
	print "Once you have your output, just copy and paste into your Arduino IDE. I'm not going to tell you how to set up your Digispark, you can handle it."

def main():
	if len(sys.argv) == 3:
		filename = sys.argv[1]
		file = open(filename)
		characters = file.read()
		file.close()
		tokens = splitpass(lexpass(characters))
		f = open(sys.argv[2], 'a')
		f.write(banner + begstr)
		for i in range(len(tokens)):
			f.write(parseblock(tokens, i))
		f.write(ending)
		f.close()
		print "Done."
		sys.exit(0)
	else:
		usage()



import lexer
from iterutils import groupby

RESERVED = 'RESERVED'
INT = 'INT'
KEY = 'KEY'
MODKEY = 'MODKEY'
STR = 'STR'
NEWLINE = 'NEWLINE'

token_exprs = [
	(r'[ \t]+',               None),
	(r'#[^\n]*',              None),
	(r'\n',                NEWLINE),
	(r'REM',              RESERVED),
	(r'DEFAULT_DELAY',    RESERVED),
	(r'DEFAULTDELAY',     RESERVED),
	(r'DELAY',            RESERVED),
	(r'STRING',           RESERVED),
	(r'GUI',                MODKEY),
	(r'WINDOWS',            MODKEY),
	(r'APP',                   KEY),
	(r'MENU',                  KEY),
	(r'SHIFT',              MODKEY),
	(r"INSERT",                KEY),
	(r'DELETE',                KEY),
	(r'HOME',                  KEY),
	(r'PAGEUP',                KEY),
	(r'PAGEDOWN',              KEY),
	(r'UPARROW',               KEY),
	(r'DOWNARROW',             KEY),
	(r'LEFTARROW',             KEY),
	(r'RIGHTARROW',            KEY),
	(r'TAB',                   KEY),
	(r'ALT',                MODKEY),
	(r'END',                   KEY),
	(r'ESC',                   KEY),
	(r'ESCAPE',                KEY),
	(r'F[1-12]',               KEY),
	(r'SPACE',                 KEY),
	(r'ENTER',                 KEY),
	(r'CONTROL',            MODKEY),
	(r'CTRL',               MODKEY),
	(r'BREAK',                 KEY),
	(r'PAUSE',                 KEY),
	(r'DOWN',                  KEY),
	(r'LEFT',                  KEY),
	(r'RIGHT',                 KEY),
	(r'UP',                    KEY),
	(r'CAPSLOCK',              KEY),
	(r'NUMLOCK',               KEY),
	(r'PRINTSCREEN',           KEY),
	(r'SCROLLLOCK',            KEY),
	(r'REPLAY',           RESERVED),
	(r'[A-Z]',                 KEY),
	(r'[0-9]+',                INT),
	(r'STRING',           RESERVED),
	(r'.*',                    STR),
	]

def lexpass(characters):
	return lexer.lex(characters,token_exprs)

def splitpass(toklist):
	l = toklist
	splitlist = [list(group) for k, group in groupby(l, lambda x: x == ("\n", "NEWLINE")) if not k]
	return splitlist



import sys
import re

def lex(characters, token_exps):
	pos = 0
	tokens = []
	while pos < len(characters):
		match = None
		for token_expr in token_exps:
			pattern, tag = token_expr
			regex = re.compile(pattern)
			match = regex.match(characters,pos)
			if match:
				text = match.group(0)
				if tag:
					token = (text, tag)
					tokens.append(token)
				break
		if not match:
			sys.stderr.write('Illegal Character: %s\n' % characters[pos])
			sys.exit(1)
		else:
			pos = match.end(0)
	return tokens


from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='digiduck',
    version='1.0.0b6',
    description='A program to compile Ducky Script to Digispark code',
    long_description=long_description,
    url='https://github.com/uslurper/digiduck',
    author='Uslurper',
    author_email='aklo101256@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Code Generators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='digispark duckyscript pentesting',
    packages=find_packages(exclude=['contrib','docs','tests']),
    install_requires=[],
    entry_points={
        'console_scripts': [
            'digiduck=digiduck:main',
        ],
    },
)
